<?php

namespace Drupal\oauthost\Entity;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\user\UserInterface;

/**
 * Defines the OAuthOST Session entity.
 *
 * @ingroup oauthost
 *
 * @ContentEntityType(
 *   id = "oauthost_session",
 *   label = @Translation("OAuthOST Session"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\oauthost\OAuthOSTSessionListBuilder",
 *     "views_data" = "Drupal\oauthost\Entity\OAuthOSTSessionViewsData",
 *
 *     "form" = {
 *       "default" = "Drupal\oauthost\Form\OAuthOSTSessionForm",
 *       "add" = "Drupal\oauthost\Form\OAuthOSTSessionForm",
 *       "edit" = "Drupal\oauthost\Form\OAuthOSTSessionForm",
 *       "delete" = "Drupal\oauthost\Form\OAuthOSTSessionDeleteForm",
 *     },
 *     "access" = "Drupal\oauthost\OAuthOSTSessionAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\oauthost\OAuthOSTSessionHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "oauthost_session",
 *   admin_permission = "administer oauthost session entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "name",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "langcode" = "langcode",
 *     "status" = "status",
 *     "authtoken" = "authtoken",
 *   },
 *   links = {
 *     "canonical" = "/admin/structure/oauthost_session/{oauthost_session}",
 *     "add-form" = "/admin/structure/oauthost_session/add",
 *     "edit-form" = "/admin/structure/oauthost_session/{oauthost_session}/edit",
 *     "delete-form" = "/admin/structure/oauthost_session/{oauthost_session}/delete",
 *     "collection" = "/admin/structure/oauthost_session",
 *   },
 *   field_ui_base_route = "oauthost_session.settings"
 * )
 */
class OAuthOSTSession extends ContentEntityBase implements OAuthOSTSessionInterface {

  use EntityChangedTrait;

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += array(
      'user_id' => \Drupal::currentUser()->id(),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isPublished() {
    return (bool) $this->getEntityKey('status');
  }

  /**
   * {@inheritdoc}
   */
  public function setPublished($published) {
    $this->set('status', $published ? TRUE : FALSE);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getAuthtoken() {
    return $this->get('authtoken')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setAuthtoken($authToken) {
    $this->set('authtoken', $authToken);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of author of the OAuthOST Session entity.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', array(
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ))
      ->setDisplayOptions('form', array(
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => array(
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ),
      ))
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('OAuthOST session id'))
      ->setDescription(t('The name of the OAuthOST Session entity.'))
      ->setSettings(array(
        'max_length' => 200,
        'text_processing' => 0,
      ))
      ->setDefaultValue('')
      ->setDisplayOptions('view', array(
        'label' => 'above',
        'type' => 'string',
        'weight' => -4,
      ))
      ->setDisplayOptions('form', array(
        'type' => 'string_textfield',
        'weight' => -4,
      ))
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

      $fields['referer'] = BaseFieldDefinition::create('string')
        ->setLabel(t('OAuthOST referer to redirect to'))
        ->setDescription(t('The OAuthOST referer to redirect to'))
        ->setSettings(array(
          'max_length' => 300,
          'text_processing' => 0,
        ))
        ->setDefaultValue('')
        ->setDisplayOptions('view', array(
          'label' => 'above',
          'type' => 'string',
          'weight' => -4,
        ))
        ->setDisplayOptions('form', array(
          'type' => 'string_textfield',
          'weight' => -4,
        ))
        ->setDisplayConfigurable('form', TRUE)
        ->setDisplayConfigurable('view', TRUE);

        $fields['configrowname'] = BaseFieldDefinition::create('string')
          ->setLabel(t('OAuthOST config row name'))
          ->setDescription(t('The OAuthOST config row name'))
          ->setSettings(array(
            'max_length' => 100,
            'text_processing' => 0,
          ))
          ->setDefaultValue('')
          ->setDisplayOptions('view', array(
            'label' => 'above',
            'type' => 'string',
            'weight' => -4,
          ))
          ->setDisplayOptions('form', array(
            'type' => 'string_textfield',
            'weight' => -4,
          ))
          ->setDisplayConfigurable('form', TRUE)
          ->setDisplayConfigurable('view', TRUE);

          $fields['authtoken'] = BaseFieldDefinition::create('string')
            ->setLabel(t('OAuthOST authtoken'))
            ->setDescription(t('The OAuthOST authtoken'))
            ->setSettings(array(
              'max_length' => 300,
              'text_processing' => 0,
            ))
            ->setDefaultValue('')
            ->setDisplayOptions('view', array(
              'label' => 'above',
              'type' => 'string',
              'weight' => -4,
            ))
            ->setDisplayOptions('form', array(
              'type' => 'string_textfield',
              'weight' => -4,
            ))
            ->setDisplayConfigurable('form', TRUE)
            ->setDisplayConfigurable('view', TRUE);

      $fields['request_token'] = BaseFieldDefinition::create('string')
        ->setLabel(t('Request Token'))
        ->setDescription(t('The Request Token.'))
        ->setSettings(array(
          'max_length' => 500,
          'text_processing' => 0,
        ))
        ->setDefaultValue('')
        ->setDisplayOptions('view', array(
          'label' => 'above',
          'type' => 'string',
          'weight' => -4,
        ))
        ->setDisplayOptions('form', array(
          'type' => 'string_textfield',
          'weight' => -4,
        ))
        ->setDisplayConfigurable('form', TRUE)
        ->setDisplayConfigurable('view', TRUE);

        $fields['request_token_secret'] = BaseFieldDefinition::create('string')
          ->setLabel(t('Request Token Secret'))
          ->setDescription(t('The Request Token Secret.'))
          ->setSettings(array(
            'max_length' => 500,
            'text_processing' => 0,
          ))
          ->setDefaultValue('')
          ->setDisplayOptions('view', array(
            'label' => 'above',
            'type' => 'string',
            'weight' => -4,
          ))
          ->setDisplayOptions('form', array(
            'type' => 'string_textfield',
            'weight' => -4,
          ))
          ->setDisplayConfigurable('form', TRUE)
          ->setDisplayConfigurable('view', TRUE);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Publishing status'))
      ->setDescription(t('A boolean indicating whether the OAuthOST Session is published.'))
      ->setDefaultValue(TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));

    return $fields;
  }

}
